package com.extrieve.quickcapture;

import android.app.Activity;
import android.content.ClipData;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Handler;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;
import com.google.gson.Gson;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.content.FileProvider;

import io.flutter.embedding.android.FlutterActivity;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

import com.extrieve.quickcapture.sdk.ImgHelper;
import com.extrieve.quickcapture.sdk.Config;
import com.extrieve.quickcapture.sdk.Config.CaptureSupport;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/** QuickcapturePlugin */
public class QuickcapturePlugin extends FlutterActivity implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native
  /// Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine
  /// and unregister it
  /// when the Flutter Engine is detached from the Activity
  private static ArrayList<String> captureimages = new ArrayList<>();
  private int REQUEST_CODE_PERMISSIONS = 1001;
  public int REQUEST_CODE_FILE_RETURN = 1002;
  private MethodChannel channel;
  private Activity activity;
  private Context context;
  private ImgHelper ImageHelper;
  private Result result;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "quickcapture");
    channel.setMethodCallHandler(this);
    context = flutterPluginBinding.getApplicationContext();

    ImageHelper = new ImgHelper(context);

    try {
      ImageHelper.Init();
    } catch (Exception e) {
      throw new RuntimeException(e);
    }

  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    this.result = result;
    switch (call.method) {
      case "initialize":
        CaptureSupport.EnableFlash = true;
        CaptureSupport.DocumentCropping = CaptureSupport.CroppingType.LiveDetection;
        result.success("QuickCapture initialised successfully");
        break;
      case "getPlatformVersion":
        result.success(Config.CaptureSupport.OutputPath);
        break;
      case "startCapture":
        Config.CaptureSupport.OutputPath = BuildStoragePath();
        this.result = result;
        OpenCameraActivity();
        break;
      case "buildTiffForLastCapture":
        result.notImplemented();
        break;
      case "buildPDFForLastCapture":
        this.result = result;
        buildPdfFileForLastCaptureSet();
        break;
      case "buildPDF":
        result.notImplemented();
        break;
      case "buildTiff":
        result.notImplemented();
        break;
      case "compressToJPEG":
        result.notImplemented();
        break;
      default:
        result.notImplemented();
        break;
    }

  }

  @Override
  public void onAttachedToActivity(ActivityPluginBinding binding) {
    activity = binding.getActivity();
    binding.addActivityResultListener((requestCode, resultCode, data) -> {
      onActivityResult(requestCode, resultCode, data);
      return false;
    });
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    activity = binding.getActivity();
    binding.addActivityResultListener((requestCode, resultCode, data) -> {
      onActivityResult(requestCode, resultCode, data);
      return false;
    });
  }

  @Override
  public void onDetachedFromActivity() {
    activity = null;
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
    // binding.removeRequestPermissionsResultListener(this);
    // binding.removeActivityResultListener(this);
  }

  protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
    super.onActivityResult(requestCode, resultCode, data);

    if (requestCode == REQUEST_CODE_FILE_RETURN && resultCode == Activity.RESULT_OK) {
      Boolean Status = (Boolean) data.getExtras().get("STATUS");
      String Description = (String) data.getExtras().get("DESCRIPTION");
      if (Status == false) {
        String imageCaptureLog = "Description : " + Description +
            ".Exception: " + Config.CaptureSupport.LastLogInfo;
        Log.d("INFO", imageCaptureLog);
        activity.finishActivity(REQUEST_CODE_FILE_RETURN);
        return;
      }
      captureimages = (ArrayList<String>) data.getExtras().get("fileCollection");
      if (captureimages == null || captureimages.isEmpty())
        return;
      OnActivitySuccess();
      // showImages(FileCollection);
      // activity.finishActivity(REQUEST_CODE_FILE_RETURN);
    }
  }

  private void OpenCameraActivity() {
    // before starting camera - configuration can set
    // String quality = ImageHelper.getCurrentImageQuality();
    try {
      // moving to camera activity in library
      Intent CameraIntent = new Intent(activity, Class.forName("com.extrieve.quickcapture.sdk.CameraHelper"));
      // photoURI = CameraSupport.CamConfigClass.OutputPath;
      Uri photoURI = Uri.parse(Config.CaptureSupport.OutputPath);
      activity.grantUriPermission(activity.getPackageName(), photoURI,
          Intent.FLAG_GRANT_WRITE_URI_PERMISSION | Intent.FLAG_GRANT_READ_URI_PERMISSION);
      if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.LOLLIPOP) {
        CameraIntent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
      }
      activity.startActivityForResult(CameraIntent, REQUEST_CODE_FILE_RETURN);
      // someActivityResultLauncher.launch(CameraIntent);

    } catch (ClassNotFoundException e) {
      Toast.makeText(activity, "Failed to open camera + ", Toast.LENGTH_LONG).show();
      e.printStackTrace();
    }
  }

  private void buildPdfFileForLastCaptureSet() {
    // runOnUiThread(new Runnable() {
    //   @Override
    //   public void run() {
    //     ShowProgressToast("building output", true);
    //   }
    // });
    AsyncTask.execute(() -> {
      // To get path from device common storage
      /*
       * Note : All apps that target Android 11 (API level 30) and above are subject
       * to Scoped Storage restrictions
       * and cannot request legacy access to device storage. Instead, they must
       * request a new permission
       * called MANAGE_EXTERNAL_STORAGE (shown to the user as “All Files Access”) to
       * be given broad storage
       * access (excluding a handful of directories like /Android/data or
       * /Android/obb).
       */
      // File sdCard = Environment.getExternalStorageDirectory();
      // File dir = new File(sdCard.getAbsolutePath() + "/QuickCapture_Tif/");

      // To get path from app internal storage
      // String path = Environment.getDataDirectory().getAbsolutePath();

      String internalAppPath = BuildStoragePath();
      // File dir = new File (path + "/QuickCapture_Tif/");
      // File dir = new File (sdCard+ "/QuickCapture_Tif/");

      File dir = new File(internalAppPath);

      if (!dir.exists())
        dir.mkdirs();
      String strPDFFile = dir + "/PDF_OUTPUT_" + UUID.randomUUID() + ".pdf";
      // String strTifFile = dir + "/TIF_OUTPUT_" + UUID.randomUUID() + ".tif";

      // String tiffFile = CameraHelper.GetTiffForLastCapture(strTifFile);
      String pdfFile = null;
      try {
        pdfFile = ImageHelper.GetPDFForLastCapture(strPDFFile);
      } catch (IOException e) {
        throw new RuntimeException(e);
      }

      if (pdfFile == null) {
        showToast("Output Not Created - Failed", Gravity.CENTER);
      } else {
        showToast("Success.Output Created", Gravity.CENTER);
        //openPdfFileTrigger(strPDFFile);
        result.success(strPDFFile);
      }
      //ShowProgressToast("building tiff output", false);
    });
  }

//  private void openPdfFileTrigger(String pdfFilePath) {
//    File file = new File(pdfFilePath);
//    // Uri path = Uri.fromFile(file);
//    String PackageName = this.getApplicationContext().getPackageName();
//    Uri photoURI = FileProvider.getUriForFile(this, PackageName + ".FileProvider", file);
//
//    Intent pdfOpenintent = new Intent(Intent.ACTION_VIEW);
//    pdfOpenintent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
//
//    List<ResolveInfo> resolvedIntentActivities = this.getPackageManager().queryIntentActivities(pdfOpenintent,
//        PackageManager.MATCH_DEFAULT_ONLY);
//    for (ResolveInfo resolvedIntentInfo : resolvedIntentActivities) {
//      String packageName = resolvedIntentInfo.activityInfo.packageName;
//      this.grantUriPermission(packageName, photoURI,
//          Intent.FLAG_GRANT_WRITE_URI_PERMISSION | Intent.FLAG_GRANT_READ_URI_PERMISSION);
//    }
//
//    pdfOpenintent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION | Intent.FLAG_GRANT_READ_URI_PERMISSION);
//
//    if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.LOLLIPOP) {
//      // Log.d(TAG, "Build <= LOLLIPOP");
//      pdfOpenintent.setClipData(ClipData.newRawUri("", photoURI));
//      pdfOpenintent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION | Intent.FLAG_GRANT_READ_URI_PERMISSION);
//    }
//
//    pdfOpenintent.setDataAndType(photoURI, "application/pdf");
//    try {
//      startActivity(pdfOpenintent);
//    } catch (ActivityNotFoundException e) {
//
//    }
//  }

  @NonNull
  private String BuildStoragePath() {
    // File direct = new File(Environment.getExternalStorageDirectory(),
    // ".GoNoGoImages");
    // return direct.getAbsolutePath();
    if (activity == null)
      return null;
    ContextWrapper c = new ContextWrapper(activity);
    String path = c.getExternalFilesDir(".QCImages").getAbsolutePath();
    return path;
  }

  /**
   * Shows a {@link Toast} on the UI thread.
   *
   * @param text The message to show
   */
  private void showToast(final String text, final int TPosition) {
    activity.runOnUiThread(() -> {
      final Toast toast = Toast.makeText(activity, text, Toast.LENGTH_SHORT);
      toast.setGravity(TPosition, 0, 10);
      toast.show();
      Handler handler = new Handler();
      handler.postDelayed(new Runnable() {
        @Override
        public void run() {
          toast.cancel();
        }
      }, 500);
    });
  }

  /* Return Section */
  class returnClass {
    ArrayList<String> fileCollection = null;
    String STATUS = "SUCCESS";
    String DESCRIPTION = "Success";
  }

  private void OnActivitySuccess() {
    returnClass dtRtn = new returnClass();
    dtRtn.fileCollection = captureimages;
    Gson gson = new Gson();
    String jsonString = gson.toJson(dtRtn);
    result.success(jsonString);
    activity.finishActivity(REQUEST_CODE_FILE_RETURN);
  }

  private void OnActivityFail(String MethodName, String FailedDescription) {
    Intent intent = new Intent();
    intent.putExtra("STATUS", false);
    String desc = "Failed : at " + MethodName;
    if (FailedDescription != null && !FailedDescription.isEmpty())
      desc += " " + FailedDescription;

    returnClass dtRtn = new returnClass();
    dtRtn.STATUS = "FAILURE";
    dtRtn.STATUS = desc;
    Gson gson = new Gson();
    String jsonString = gson.toJson(dtRtn);
    result.success(jsonString);
    activity.finishActivity(REQUEST_CODE_FILE_RETURN);
  }

}
